#include "Chart.h"

// The x position that the x axis start from.
#define BOTTOM_AXIS_X_ 74

// The y position that the x axis start from.
#define BOTTOM_AXIS_Y_ 170

// The x position for the vertical axis.
#define VERTICAL_AXIS_X_ 58

// The y position for the bottom of the vertical axis.
#define VERTICAL_AXIS_Y_BOTTOM_ 158

// The number of fish that a relevant for showing on the chart.
#define CHART_FISH_TYPE_COUNT_ 8

// The separation between sprites and bars on the chart.
const u8 barSeparation = TILE_WIDTH_BYTES * 4 + 2;

void displayFishChart(ALLEGRO_EVENT_QUEUE** eventQueue)
{
    al_clear_to_color(al_map_rgb(0, 0, 128));
    ALLEGRO_COLOR textColour = al_map_rgb(255, 255, 255);

    // Clear the screen
    al_clear_to_color(al_map_rgb(0, 0, 128));

    // Title of the screen.
    drawString(104 * scaleFactor, 30 * scaleFactor, "RESULTS", textColour);

    // Draws the bottom axis with the fish sprites.
    drawBottomAxis();

    // Draws the bars on the chart.
    drawDataBars();

    // Draws the vertical axis with the numbers on the left side of the chart.
    drawVerticalAxis();

    al_flip_display();
    getKey(eventQueue);
}

void drawBottomAxis()
{
    u8 i = 0;
    u8 x = BOTTOM_AXIS_X_;
    u8 y = BOTTOM_AXIS_Y_;

    // Only 8 of the fish types are drawn on the chart.
    for (i = 0; i < CHART_FISH_TYPE_COUNT_; ++i)
    {
        al_draw_scaled_bitmap(allegroEnemySprites[i], 0, 0, TILE_WIDTH_BYTES * 2, TILE_HEIGHT, x * scaleFactor, y * scaleFactor, TILE_WIDTH_BYTES * 4 * scaleFactor, TILE_HEIGHT * scaleFactor, 0);

        x += barSeparation;
    }
}

u8 playerHasEaten(u8 type)
{
    u8 i = 0;
    u8 count = 0;

    for (i = 0; i < player.fishEaten; ++i)
    {
        if (fishEaten[i] == type)
        {
            ++count;
        }
    }

    return count;
}

void drawDataBars()
{
    u8 i = 0;

    u8 barColours[CHART_FISH_TYPE_COUNT_] = { 6, 18, 10, 11, 15, 23, 24, 0 };

    for (i = 0; i < CHART_FISH_TYPE_COUNT_; ++i)
    {
        // Only fish that the player has eaten will have a bar drawn.
        u8 count = playerHasEaten(i);

        if (count)
        {
            u8 y2 = (BOTTOM_AXIS_Y_ - 1);
            u8 height = count * TILE_HEIGHT;
            u8 y1 = y2 - height;
            u8 x1 = (BOTTOM_AXIS_X_ + barSeparation * i);
            u8 width = TILE_WIDTH_BYTES * 4;
            u8 x2 = x1 + width;

            al_draw_filled_rectangle(x1 * scaleFactor, y1 * scaleFactor, x2 * scaleFactor, y2 * scaleFactor, firmwareToAllegroColour(barColours[i]));
        }
    }
}

void drawVerticalAxis()
{
    u8 x = VERTICAL_AXIS_X_;
    u8 y = VERTICAL_AXIS_Y_BOTTOM_;
    u8 i = 0;
    u8 text[2];

    for (i = 0; i < MAX_FISH_CAN_EAT; ++i)
    {
        convertNumberToString(i + 1, text);
        drawString(x * scaleFactor, y * scaleFactor, text, al_map_rgb(255, 255, 255));

        y -= TILE_HEIGHT;
    }
}