#ifndef ENEMY_H
#define ENEMY_H

#include "Actor.h"
#include "GameTypes.h"
#include "Enemy.h"
#include "Map.h"
#include "Object.h"
#include "SpriteSets.h"
#include "Utility.h"

typedef struct Enemy Enemy;

struct Enemy
{
    // Actor object containing position and velocity.
    Object object;

    // Number of hitpoints.
    u8 health;

    // Indicates whether the actor should be redrawn on screen.
    u8 redraw;

    // The type of enemy.
    u8 type;

    // For reducing the frequency that the enemy logic runs thus improving performance (also allows for speed adjustments).
    u8 logicFlag;

    // Index of the enemy in the array.
    u8 id;
};

// TODO real value, Maximum number of enemies that can be loaded for a room.
#define MAX_ROOM_ENEMY_COUNT 15

// The number of enemies in the room.
extern u8 roomEnemyCount;

// The enemy entities spawned in the room.
extern Enemy roomEnemies[MAX_ROOM_ENEMY_COUNT];

// Clears all the enemies making it ready for the next game.
void clearEnemies();

// Adds an enemy of the specified enemy type.
void addEnemy(u8 x, u8 y, u8 type);

// Initialises the enemy ready for drawing on screen.
void initEnemy(Enemy* enemy, u8 x, u8 y, u8 type);

// Retrieves the next index in the enemy array for inserting an element into the enemy array (will return 255 if full).
u8 nextEnemyIndex();

// Draw the enemies in spawned in the current room on screen.
void drawEnemies();

// Forces a redraw of all the enemies in the room.
void redrawAllEnemies();

// Run enemy logic, performing enemy actions and moving.
void logicEnemies();

// Update a specified enemy's position based on velocity and whether it is impeded (collision detection).
void updateEnemyPosition(Enemy* enemy);

// Processes the cell and indicates whether it is solid (can the enemy pass through).
u8 processCellForEnemy(u8 x, u8 y);

// The amount of score received for eating different types of fish.
u16 fishScore(u8 type);

#endif