#ifndef GAME_H
#define GAME_H

#include <allegro5/allegro.h>
#include "Dialog.h"
#include "Enemy.h"
#include "GameState.h"
#include "GameTypes.h"
#include "Hud.h"
#include "Keys.h"
#include "Map.h"
#include "Player.h"
#include "SpriteSets.h"

// Runs the full game session and manages the levels in the game.
void runGameSession(u8 fpsCap);

// Runs the game and the game loop with allegro.
void runGame(u8 fpsCap);

// Displays the contents of the game on the screen as it changes.
void drawGame();

// Read input from the player from the allegro event.
void readInput(ALLEGRO_EVENT_QUEUE* eventQueue, ALLEGRO_EVENT* allegroEvent);

// This is where the processing of all the game logic happens.
void gameLogic();

// Initialises the ALLEGRO_TIMERs used in the game.
void initialiseAllAllegroTimers();

// Destroys all the ALLEGRO_TIMERs used in the game.
void destroyAllAllegroTimers();

// Checks the state of the ALLEGRO_TIMERs used in the game.
void checkTimers();

// Randomly spawns new fish for the player to consider eating.
void spawnFish();

// Calculate the next set of coordinates for spawning a fish.
void nextFishCoordinates(u8* x, u8* y);

// Randomly determine the next fish type, some fish are rarer than others.
u8 nextFishType();

#endif