#ifndef GAMETYPES_H
#define GAMETYPES_H

// Boolean true value 1.
#define TRUE 1

// Boolean false value 0.
#define FALSE 0

// The number of cells in the map grid.
#define MAP_CELL_COUNT 2160

// Number of tiles that make up the level vertically.
#define MAP_TILES_Y 12

// Number of tiles that make up the level horizontally.
#define MAP_TILES_X 20

// Number of tiles that make up the screen vertically.
#define SCREEN_TILES_Y 12

// Number of tiles that make the screen horizontally.
#define SCREEN_TILES_X 20

// Height of a tile on screen.
#define TILE_HEIGHT 16

// Width of a tile on screen.
#define TILE_WIDTH_BYTES 4

// The possible bits that the cell graphic could be in.
#define CELL_GFX_RANGE 0x1F

// The length of time for a level, somewhat arbitrary may change this.
#define MAX_LEVEL_TIME 30

// The amount to scale the sprites based on screen resolution.
extern unsigned char scaleFactor;

// Cpctelera defines shortcut types in types.h that we are defining here for when using allegro.
typedef unsigned char       u8;
typedef signed char         i8;
typedef unsigned short      u16;
typedef signed short        i16;
typedef unsigned long      u32;
typedef signed long        i32;
typedef unsigned long long u64;
typedef signed long long   i64;
typedef float              f32;

// The number of levels in the game.
#define LEVEL_COUNT 3

// Ready to start the level.
#define LEVEL_READY 0

// The player has eaten the max number of fish.
#define LEVEL_FULL 1

// The level has been exited via the exit dialog.
#define LEVEL_EXIT 2

// The level timer has run out.
#define LEVEL_TIME_OUT 3

// Right direction.
#define DIRECTION_RIGHT 0

// Left direction.
#define DIRECTION_LEFT 1

// Screen X dimension.
// Note: 80 = 160/2
#define SCREEN_X 80

// Screen y dimension only including tiles (hud height isn't included).
#define SCREEN_Y_NO_HUD 192

// Cell empty flag.
#define CELL_EMPTY 0

// Maximum number of fish the player can eat.
#define MAX_FISH_CAN_EAT 8

#endif