#include "Map.h"

// The map grid.
u8 mapCells[MAP_CELL_COUNT];

// Determine when a map redraw should happen.
u8 mapRedraw = TRUE;

// The map index of the current map in the maps array.
u8 currentMap = 0;

// The number of enemies in the room.
u8 roomEnemyCount;

// The enemy entities spawned in the room.
Enemy roomEnemies[MAX_ROOM_ENEMY_COUNT];

void initMap()
{
    memcpy(mapCells, level_map_uncompressed[currentMap], MAP_CELL_COUNT);

    // Enable map redraw.
    mapRedraw = TRUE;
}

void drawMap()
{
    if (mapRedraw)
    {
        // Draw the content of the map grid.
        u8 x;
        u8 y;
        u8 cell;

        // Assuming the game is only a single screen for now.
        u8 currentRoomX = 0;
        u8 currentRoomY = 0;

        // Position is determined by how much the screen has scrolled to show the current room.
        u16 position = mapCellPosition(0, 0, currentRoomX, currentRoomY);
        u8* cellPointer = mapCells + position;

        // Position is the element in the map tile array.
        for (x = 0; x < SCREEN_TILES_X; ++x)
        {
            for (y = 0; y < SCREEN_TILES_Y; ++y)
            {
                cell = *cellPointer;
                drawCell(x, y, cell);

                ++cellPointer;
            }

            // Not required in a 1 room map, will come back to this if necessary.
            // Move to the start of the next column (half the column height in a 2x2 screen map), 2/3 of the column height in a 3x3 map.
            //cellPointer += 24;
        }

        mapRedraw = FALSE;
    }
}

u16 mapCellPosition(u8 x, u8 y, u8 roomX, u8 roomY)
{
    return (x + (SCREEN_TILES_X * roomX)) * MAP_TILES_Y + (y + (SCREEN_TILES_Y * roomY));
}

u8 mapCell(u8 x, u8 y, u8 roomX, u8 roomY)
{
    return mapCells[mapCellPosition(x, y, roomX, roomY)];
}

void drawCell(u8 x, u8 y, u8 cell)
{
    u16 targetX = x * 16 * scaleFactor;
    u16 targetY = y * 16 * scaleFactor;
    ALLEGRO_BITMAP* bitmap = allegroBitmapFromCell(cell);

    if (bitmap)
    {
        al_draw_scaled_bitmap(bitmap, 0, 0, TILE_WIDTH_BYTES * 2, TILE_HEIGHT, targetX, targetY, 16 * scaleFactor, 16 * scaleFactor, 0);
    }
}

ALLEGRO_BITMAP* allegroBitmapFromCell(u8 cell)
{
    // The cell type without the flag.
    u8 cellType = cell & CELL_GFX_RANGE;

    if (cell != 129)
    {
        return allegroTileSprites[cellType - 1];
    }
    else
    {
        // 129 is being used as the empty cell value in the editor.
        return allegroTileSprites[13];
    }
}