#ifndef PLAYER_H
#define PLAYER_H

#include "Actor.h"
#include "Enemy.h"
#include "GameTypes.h"
#include "Hud.h"
#include "Keys.h"
#include "Object.h"
#include "SpriteSets.h"

// The default height of the player.
#define PLAYER_HEIGHT 32

// The default width of the player.
#define PLAYER_WIDTH TILE_WIDTH_BYTES

typedef struct Player Player;

struct Player
{
    // Actor object containing position and velocity.
    Object object;

    // Number of hitpoints.
    u8 health;

    // Indicates whether the actor should be redrawn on screen.
    u8 redraw;

    // The direction that the player is currently facing.
    u8 direction;

    // The number of fish the player has eaten.
    u8 fishEaten;
};

// Global player object.
extern Player player;

// Initialises the player struct with initial values.
void initPlayer(u8 x, u8 y);

// Draws the player on the screen as positioned in the player object.
void drawPlayer();

// Updates the player velocity based on user input.
void movePlayer();

// Player logic such as interactions and velocity/position.
void logicPlayer();

// Checks if a player has collided with an enemy fish and whether it eats the enemy.
void playerEnemyCollision();

#endif