#include "SpriteSets.h"

u8* const tileSprites[TILE_SPRITE_COUNT] =
{
    G_tile_grass_0,
    G_tile_river_edge_0,
    G_tile_river_edge_1,
    G_tile_river_edge_2,
    G_tile_river_edge_3,
    G_tile_river_edge_4,
    G_tile_river_edge_5,
    G_tile_river_edge_6,
    G_tile_river_edge_7,
    G_tile_river_edge_8,
    G_tile_river_edge_9,
    G_tile_river_edge_10,
    G_tile_river_edge_11,
    G_tile_water
};

u8* const playerSprites[PLAYER_SPRITE_COUNT] =
{
    G_heron_3_flying_right,
    G_heron_3_flying_left
};

u8* const enemySprites[ENEMY_SPRITE_COUNT] =
{
    G_enemy_slug,
    G_enemy_fish_1,
    G_enemy_fish_2,
    G_enemy_fish_3,
    G_enemy_fish_4,
    G_enemy_fish_5,
    G_enemy_octopus,
    G_enemy_crab,
    G_enemy_snail,
    G_enemy_snail_2
};

// Array for accessing hud sprites in CPC format (not actually used from CPC code).
u8* const hudSprites[HUD_SPRITE_COUNT] =
{
    G_hud_fish,
};

// Array for accessing tile sprites in Allegro.
ALLEGRO_BITMAP* allegroTileSprites[TILE_SPRITE_COUNT];

// Array for accessing player sprites in Allegro.
ALLEGRO_BITMAP* allegroPlayerSprites[PLAYER_SPRITE_COUNT];

// Array for accessing enemy sprites in Allegro.
ALLEGRO_BITMAP* allegroEnemySprites[ENEMY_SPRITE_COUNT];

// Array for accessing hud sprites in Allegro.
ALLEGRO_BITMAP* allegroHudSprites[HUD_SPRITE_COUNT];

void initAllAllegroSpriteSets()
{
    initAllegroSpriteSet(allegroTileSprites, tileSprites, TILE_WIDTH_BYTES, TILE_HEIGHT, TILE_SPRITE_COUNT);
    initAllegroSpriteSet(allegroPlayerSprites, playerSprites, PLAYER_WIDTH, PLAYER_HEIGHT, PLAYER_SPRITE_COUNT);
    initAllegroSpriteSet(allegroEnemySprites, enemySprites, TILE_WIDTH_BYTES, TILE_HEIGHT, ENEMY_SPRITE_COUNT);
    initAllegroSpriteSet(allegroHudSprites, hudSprites, 5, 8, HUD_SPRITE_COUNT);
}

void initAllegroSpriteSet(ALLEGRO_BITMAP** allegroBitmaps, u8** sprites, u8 width, u8 height, u8 count)
{
    u8 i = 0;

    // The palette defining the colours that will be used to draw the graphics in the game.
    const u8 gamePalette[16] = { 0x4, 0xa, 0x13, 0xc, 0xb, 0x14, 0x15, 0xd, 0x6, 0x1e, 0x1f, 0x7, 0x12, 0x19, 0x0, 0x16 };

    for (i = 0; i < count; ++i)
    {
        allegroBitmaps[i] = constructBitmap(sprites[i], width * height, width, height, width * 2, gamePalette);
    }
}

void destroyAllAllegroSpriteSets()
{
    destroyAllegroSpriteSet(allegroTileSprites, TILE_SPRITE_COUNT);
    destroyAllegroSpriteSet(allegroPlayerSprites, PLAYER_SPRITE_COUNT);
    destroyAllegroSpriteSet(allegroEnemySprites, ENEMY_SPRITE_COUNT);
}

void destroyAllegroSpriteSet(ALLEGRO_BITMAP** sprites, u8 count)
{
    u8 i = 0;

    for (i = 0; i < count; ++i)
    {
        al_destroy_bitmap(sprites[i]);
    }
}