#ifndef SPRITESETS_H
#define SPRITESETS_H

#include <allegro5/allegro.h>
#include "GameTypes.h"
#include "Player.h"
#include "SpriteUtil.h"
#include "sprites/TileSprites.h"
#include "sprites/PlayerSprites.h"
#include "sprites/EnemySprites.h"
#include "sprites/HudSprites.h"

// The number of tile sprites in the tile sprite set.
#define TILE_SPRITE_COUNT 14

// The number of player sprites in the player sprite set.
#define PLAYER_SPRITE_COUNT 2

// The number of enemy sprites in the enemy sprite set.
#define ENEMY_SPRITE_COUNT 10

// The number of hud sprites in the hud sprite set.
#define HUD_SPRITE_COUNT 1

// Array for accessing tile sprites in CPC format.
extern u8* const tileSprites[TILE_SPRITE_COUNT];

// Array for accessing tile sprites in Allegro.
extern ALLEGRO_BITMAP* allegroTileSprites[TILE_SPRITE_COUNT];

// Array for accessing player sprites in Allegro.
extern ALLEGRO_BITMAP* allegroPlayerSprites[PLAYER_SPRITE_COUNT];

// Array for accessing enemy sprites in Allegro.
extern ALLEGRO_BITMAP* allegroEnemySprites[ENEMY_SPRITE_COUNT];

// Array for accessing hud sprites in CPC format (not actually used from CPC code).
extern u8* const hudSprites[HUD_SPRITE_COUNT];

// Array for accessing hud sprites in Allegro.
extern ALLEGRO_BITMAP* allegroHudSprites[HUD_SPRITE_COUNT];

// Initialise the all the sprites into ALLEGRO_BITMAP objects.
void initAllAllegroSpriteSets();

// Initialise the tile size sprites into ALLEGRO_BITMAP objects.
void initAllegroSpriteSet(ALLEGRO_BITMAP** allegroBitmaps, u8** sprites, u8 width, u8 height, u8 count);

// Destroys all the ALLEGRO_BITMAP objects in the allegro sprites array.
void destroyAllAllegroSpriteSets();

// Destroys all the ALLEGRO_BITMAP in the specified array.
void destroyAllegroSpriteSet(ALLEGRO_BITMAP** sprites, u8 count);

#endif