#include "Text.h"

ALLEGRO_FONT* defaultFont_ = NULL;

void initFonts(const char* fontPath)
{
    defaultFont_ = al_load_font(fontPath, 8 * scaleFactor, 0);
}

void drawString(u16 x, u16 y, const char* string, ALLEGRO_COLOR textColour)
{
    al_draw_text(defaultFont_, textColour, x, y, 0, string);
}

void destroyFonts()
{
    al_destroy_font(defaultFont_);
}

void drawScore(u16 score, u16 x, u16 y, ALLEGRO_COLOR textColour)
{
    // Pad the score with 0s and multiply by 10 to get on screen value.
    static unsigned char scoreText[7];
#ifdef _MSC_VER
    sprintf_s(scoreText, 7, "%06d", score * 10);
#else
    sprintf(scoreText, "%06d", score * 10);
#endif
    drawString(x, y, scoreText, textColour);
}

void drawTimeRemaining(u8 time, u16 xPosition, u16 yPosition, ALLEGRO_COLOR textColour)
{
    // Calculates the actual amount of time remaining.
    u8 remainingTime = MAX_LEVEL_TIME - time;
    static u8 timeText[4];

#ifdef _MSC_VER
    sprintf_s(timeText, 4, "%03d", remainingTime);
#else
    sprintf(timeText, "%03d", remainingTime);
#endif

    // Draw the actual time remaining on the hud.
    drawString(xPosition, yPosition, timeText, textColour);
}

void convertNumberToString(u16 input, u8* output)
{
    // Need to add 48 to each number to actually display the correct number on screen.
    u8 nextDigit = FALSE;

    if (input >= 10000)
    {
        *output = (input / 10000) + ASCII_TO_NUMBER_OFFSET;
        ++output;
        input = (input % 10000);
        nextDigit = TRUE;
    }

    if (input >= 1000 || nextDigit)
    {
        *output = (input / 1000) + ASCII_TO_NUMBER_OFFSET;
        ++output;
        input = (input % 1000);
        nextDigit = TRUE;
    }

    if (input >= 100 || nextDigit)
    {
        *output = (input / 100) + ASCII_TO_NUMBER_OFFSET;
        ++output;
        input = (input % 100);
        nextDigit = TRUE;
    }

    if (input >= 10 || nextDigit)
    {
        *output = (input / 10) + ASCII_TO_NUMBER_OFFSET;
        ++output;
        input = (input % 10);
        nextDigit = TRUE;
    }

    if (input < 10 || nextDigit)
    {
        *output = input + ASCII_TO_NUMBER_OFFSET;
        ++output;
    }

    // Add the null terminator at the end.
    *output = '\0';
}